﻿' ResultsTools.vb - Collection of routines for querying and accessing results
'
'   History: 
'    20-Dec-2010 / Jim Bernard / created / NX 7.5
'
'   Description:
'
Option Strict Off
Imports System

Imports NXOpen
Imports NXOpen.UF

Module ResultsTools

    Structure resultMenuOptions
        Dim simFileName As String
        Dim selectedSolution As Integer
        Dim selectedLoadcase As Integer
        Dim selectedLoadcases() As Integer
        Dim selectedIterations() As Integer
        Dim selectedResultType As Integer

        Dim solutions() As solutionMenuOptions
        Dim solutionNames() As String
    End Structure

    Structure solutionMenuOptions
        Dim solution As CAE.SimSolution
        Dim result As CAE.Result
        Dim loadcases() As loadcaseMenuOptions
        Dim loadcaseNames() As String
        Dim maxIterationsPerLoadcase As Integer
    End Structure

    Structure loadcaseMenuOptions
        Dim loadcase As CAE.Loadcase
        Dim iterations() As iterationMenuOptions
        Dim iterationNames() As String
    End Structure

    Structure iterationMenuOptions
        Dim iteration As CAE.Iteration
        Dim resultTypes() As CAE.ResultType
        Dim resultTypeNames() As String
    End Structure

    Function getResultMenuOptions(ByVal solutions() As CAE.SimSolution, ByVal resQuantity As CAE.Result.Quantity) As resultMenuOptions

        ' This function drills down through the available results for each input solution and
        ' populates a structure to be used for driving results selection options on a
        ' user defined dialog.

        Dim numSolutions As Integer = solutions.Length - 1

        Dim resMenOpt As New resultMenuOptions
        ReDim resMenOpt.solutions(numSolutions)
        ReDim resMenOpt.solutionNames(numSolutions)
        For iSolution As Integer = 0 To numSolutions

            resMenOpt.solutions(iSolution).solution = solutions(iSolution)
            resMenOpt.solutionNames(iSolution) = resMenOpt.solutions(iSolution).solution.Name
            resMenOpt.solutions(iSolution).result = getSolutionResults(solutions(iSolution))

            Dim baseLoadcases() As CAE.BaseLoadcase = resMenOpt.solutions(iSolution).result.GetLoadcases()
            Dim numLoadcases As Integer = baseLoadcases.Length - 1
            ReDim resMenOpt.solutions(iSolution).loadcases(numLoadcases)
            ReDim resMenOpt.solutions(iSolution).loadcaseNames(numLoadcases)

            For iLoadcase As Integer = 0 To numLoadcases

                resMenOpt.solutions(iSolution).loadcases(iLoadcase).loadcase = CType(baseLoadcases(iLoadcase), CAE.Loadcase)
                resMenOpt.solutions(iSolution).loadcaseNames(iLoadcase) = resMenOpt.solutions(iSolution).loadcases(iLoadcase).loadcase.Name

                Dim baseIterations() As CAE.BaseIteration = baseLoadcases(iLoadcase).GetIterations
                Dim numIterations As Integer = baseIterations.Length - 1
                ReDim resMenOpt.solutions(iSolution).loadcases(iLoadcase).iterations(numIterations)
                ReDim resMenOpt.solutions(iSolution).loadcases(iLoadcase).iterationNames(numIterations)
                If numIterations > resMenOpt.solutions(iSolution).maxIterationsPerLoadcase Then resMenOpt.solutions(iSolution).maxIterationsPerLoadcase = numIterations

                For iIter As Integer = 0 To numIterations

                    resMenOpt.solutions(iSolution).loadcases(iLoadcase).iterations(iIter).iteration = CType(baseIterations(iIter), CAE.Iteration)
                    resMenOpt.solutions(iSolution).loadcases(iLoadcase).iterationNames(iIter) = resMenOpt.solutions(iSolution).loadcases(iLoadcase).iterations(iIter).iteration.Name

                    Dim baseResultTypes() As CAE.BaseResultType = baseIterations(iIter).GetResultTypes
                    Dim numResultTypes As Integer = baseResultTypes.Length - 1
                    ReDim resMenOpt.solutions(iSolution).loadcases(iLoadcase).iterations(iIter).resultTypes(numResultTypes)
                    ReDim resMenOpt.solutions(iSolution).loadcases(iLoadcase).iterations(iIter).resultTypeNames(numResultTypes)

                    Dim resTypeCnt As Integer = 0
                    For Each baseResType As CAE.BaseResultType In baseResultTypes

                        If baseResType.Quantity = resQuantity Then

                            resMenOpt.solutions(iSolution).loadcases(iLoadcase).iterations(iIter).resultTypes(resTypeCnt) = CType(baseResType, CAE.ResultType)
                            resMenOpt.solutions(iSolution).loadcases(iLoadcase).iterations(iIter).resultTypeNames(resTypeCnt) = resMenOpt.solutions(iSolution).loadcases(iLoadcase).iterations(iIter).resultTypes(resTypeCnt).Name
                            resTypeCnt += 1

                        End If

                    Next baseResType

                    If resTypeCnt = 0 Then

                        ''theNXMessageBox.Show(MessageBoxTitle, NXMessageBox.DialogType.Error, "No " + resQuantity.ToString + " Results for Solution: " + resMenOpt.solutions(iSolution).solution.Name + "; Load Case: " + resMenOpt.solutions(iSolution).loadcases(iLoadcase).loadcase.Name + "; Iteration: " + resMenOpt.solutions(iSolution).loadcases(iLoadcase).iterations(iIter).iteration.Name)

                    Else

                        ReDim Preserve resMenOpt.solutions(iSolution).loadcases(iLoadcase).iterations(iIter).resultTypes(resTypeCnt - 1)
                        ReDim Preserve resMenOpt.solutions(iSolution).loadcases(iLoadcase).iterations(iIter).resultTypeNames(resTypeCnt - 1)

                    End If

                Next iIter

            Next iLoadcase

        Next iSolution

        Return resMenOpt

    End Function

    Function getSolutionsWithSpecifiedResultType(ByVal sim As CAE.SimSimulation, ByVal resQuantity As CAE.Result.Quantity) As CAE.SimSolution()

        ' This function returns an array of all SimSolutions in the
        ' specified SimSimulation that contain Strain or Kinetic Energy results

        Dim simSolutions() As CAE.SimSolution = sim.Solutions.ToArray

        Dim solutionsWithResults As New List(Of CAE.SimSolution)

        If (simSolutions Is Nothing) Then
            theNXMessageBox.Show(MessageBoxTitle, NXMessageBox.DialogType.Error, "No Solutions exist")
            Return Nothing
        End If

        For Each simSolution As CAE.SimSolution In simSolutions

            Dim result As CAE.Result = getSolutionResults(simSolution)

            If Not result Is Nothing Then
                For iLc As Integer = 0 To result.AskNumLoadcases - 1
                    For iIt As Integer = 0 To result.AskNumIterations(iLc) - 1
                        Dim resTypes() As CAE.Result.Type = Nothing
                        Dim resTypeDesc() As String = Nothing
                        result.AskResultTypes(iLc, iIt, resTypes, resTypeDesc)
                        For Each resType As CAE.Result.Type In resTypes
                            If resType.Quantity = resQuantity Then
                                If Not solutionsWithResults.Contains(simSolution) Then solutionsWithResults.Add(simSolution)
                            End If
                        Next resType
                    Next iIt
                Next iLc
            End If

        Next simSolution

        If solutionsWithResults.Count = 0 Then
            Return Nothing
        Else
            Return solutionsWithResults.ToArray
        End If

    End Function

    Function getSolutionResults(ByVal solution As CAE.SimSolution) As CAE.Result

        ' This function returns the Result object for a given SimSolution

        Dim results As CAE.Result = Nothing

        Dim theResultManager As CAE.ResultManager = theSession.ResultManager()

        If (theResultManager Is Nothing) Then
            theNXMessageBox.Show(MessageBoxTitle, NXMessageBox.DialogType.Error, "No valid ResultManager")
            Return Nothing
        End If

        Dim solresults As CAE.SolutionResult = Nothing

        Try

            solresults = theResultManager.CreateSolutionResult(solution)

        Catch ex As Exception
            ''theNXMessageBox.Show(MessageBoxTitle, NXMessageBox.DialogType.Error, ex.ToString)
            ''Return Nothing
        End Try

        If (solresults Is Nothing) Then
            ''theNXMessageBox.Show(MessageBoxTitle, NXMessageBox.DialogType.Error, "Could not Access Solution Results")
            Return Nothing
        End If

        results = CType(solresults, CAE.Result)

        Return results

    End Function

    'Function getElementResultAccess(ByVal result As CAE.Result, ByVal resultType As CAE.ResultType, ByVal iLoadcase As Integer, ByVal iIteration As Integer, ByVal component As CAE.Result.Component) As CAE.ResultAccess
    Function getElementResultAccess(ByVal result As CAE.Result, ByVal resultType As CAE.ResultType, ByVal component As CAE.Result.Component) As CAE.ResultAccess

        ' This function returns a ResultAcces object for the specified component.
        ' The returned ResultAccess object will have node averaging set to on across all options.

        Dim MessageBoxTitle As String = "getElementResultAccess"

        ' Create results access
        Dim resultAccess As CAE.ResultAccess = Nothing

        Try
            Dim resultParameters As CAE.ResultParameters = theSession.ResultManager.CreateResultParameters

            ''resultParameters.SetLoadcaseIteration(iLoadcase, iIteration)
            ''resultParameters.SetResultType(resultType)
            resultParameters.SetGenericResultType(resultType)
            resultParameters.SetResultComponent(component)

            resultAccess = theSession.ResultManager.CreateResultAccess(result, resultParameters)

            theSession.ResultManager.DeleteResultParameters(resultParameters)

        Catch ex As Exception

            theNXMessageBox.Show(MessageBoxTitle, NXMessageBox.DialogType.Error, ex.ToString)
            Return Nothing

        End Try

        Return resultAccess

    End Function

End Module
