﻿
Option Strict Off
Imports System
Imports NXOpen
Imports NXOpen.UF

Module TraverseAssyFemGroups

    Dim theLWPrefix As String = ""

    '  Explicit Activation
    '      This entry point is used to activate the application explicitly
    Function getCaeGroups(ByVal caePart As CAE.CaePart) As CAE.CaeGroup()

        If (caePart Is Nothing) Then Return Nothing

        Dim iMeshManager As CAE.IMeshManager = Nothing

        Dim feModelOccList As New List(Of CAE.FEModelOccurrence)
        Dim baseFeModel As CAE.BaseFEModel = Nothing

        Dim baseFemPart As CAE.BaseFemPart = Nothing
        Dim assyFemPart As CAE.AssyFemPart = Nothing
        Dim simPart As CAE.SimPart = Nothing

        If TypeOf caePart Is CAE.BaseFemPart Then

            baseFemPart = CType(caePart, CAE.BaseFemPart)
            baseFeModel = baseFemPart.BaseFEModel

        ElseIf TypeOf caePart Is CAE.AssyFemPart Then

            assyFemPart = CType(caePart, CAE.AssyFemPart)
            Dim assyFEModel As CAE.AssyFEModel = CType(assyFemPart.BaseFEModel, CAE.AssyFEModel)

            If assyFEModel IsNot Nothing Then feModelOccList.AddRange(assyFEModel.GetChildren)

        ElseIf TypeOf caePart Is CAE.SimPart Then

            simPart = CType(caePart, CAE.SimPart)
            feModelOccList.Add(simPart.Simulation.Femodel)

        End If

        If DEBUG Then
            theLW.Open()
            theLW.WriteLine(theLWPrefix + "Start of test case 1 " + vbCrLf)
        End If

        Dim caeGroups As New List(Of CAE.CaeGroup)
        caeGroups.AddRange(caePart.CaeGroups.ToArray)

        Dim caeOccGroups As New List(Of CAE.CaeGroup)

        If feModelOccList IsNot Nothing Then
            For Each feModelOcc As CAE.FEModelOccurrence In feModelOccList
                caeOccGroups.AddRange(processFEModelOccurrence(feModelOcc))
            Next feModelOcc
        Else

            If DEBUG Then theLW.WriteLine(theLWPrefix + "There are no FEModelOccs at this level " + vbLf)

        End If

        For Each caeOccGroup As CAE.CaeGroup In caeOccGroups
            If Not caeGroups.Contains(caeOccGroup) Then caeGroups.Add(caeOccGroup)
        Next caeOccGroup

        If DEBUG Then theLW.WriteLine(vbCrLf + theLWPrefix + "End of test case 1 ")

        Return caeGroups.ToArray

    End Function

    Function processFEModelOccurrence(ByRef feModelOcc As CAE.FEModelOccurrence) As List(Of CAE.CaeGroup)

        Dim caeGroups As New List(Of CAE.CaeGroup)

        Dim prevPrefix As String = theLWPrefix
        theLWPrefix = theLWPrefix + vbTab
        If DEBUG Then
            theLW.WriteLine(vbCrLf + theLWPrefix + "processFEModelOccurrence Start" + vbCrLf + vbCrLf)
            theLW.WriteLine(theLWPrefix + "Input FEModelOccurrence[" + feModelOcc.Tag.ToString + "] has ")
            theLW.WriteLine(theLWPrefix + "Owning Part = " + feModelOcc.OwningPart.FullPath)
        End If

        Dim prototypeBaseFemPart As CAE.BaseFemPart = Nothing
        Dim prototypeBaseFEModel As CAE.IFEModel = Nothing

        If (feModelOcc.GetComponent IsNot Nothing) Then
            Dim comp As Assemblies.Component = feModelOcc.GetComponent
            Dim inxobj As INXObject = comp.Prototype

            If inxobj IsNot Nothing Then

                Dim objType As String = "No Type "

                Dim objTag As Tag = Tag.Null
                If TypeOf comp.Prototype Is TaggedObject Then

                    objTag = (CType(comp.Prototype, TaggedObject)).Tag
                    objType = (CType(comp.Prototype, TaggedObject)).GetType.ToString
                End If

                If TypeOf comp.Prototype Is CAE.BaseFemPart Then
                    prototypeBaseFemPart = CType(comp.Prototype, CAE.BaseFemPart)
                    prototypeBaseFEModel = prototypeBaseFemPart.BaseFEModel
                End If

                If TypeOf comp.Prototype Is CAE.CaePart Then
                    Dim prototypeCaePart As CAE.CaePart = CType(comp.Prototype, CAE.CaePart)
                    Dim protoCaeGroups As CAE.CaeGroupCollection = prototypeCaePart.CaeGroups

                    For Each protoCaeGroup As CAE.CaeGroup In protoCaeGroups
                        If DEBUG Then theLW.WriteLine(theLWPrefix + "Cae Group = [" + protoCaeGroup.Name + "] tag = " + protoCaeGroup.Tag.ToString + " part = " + protoCaeGroup.OwningPart.FullPath)

                        Dim femOccCaeGroup As CAE.CaeGroup = Nothing
                        Try
                            femOccCaeGroup = CType(feModelOcc.FindObject("CaeGroup[" + protoCaeGroup.Name + "]"), CAE.CaeGroup)
                        Catch ex As Exception

                        End Try

                        If femOccCaeGroup IsNot Nothing Then
                            If DEBUG Then theLW.WriteLine(theLWPrefix + "Cae Group Occ = [" + femOccCaeGroup.Name + "] tag = " + femOccCaeGroup.Tag.ToString + " part = " + femOccCaeGroup.OwningPart.FullPath)
                            caeGroups.Add(femOccCaeGroup)
                        Else
                            If DEBUG Then theLW.WriteLine(theLWPrefix + "Cae Group  Occ not found ")
                        End If

                    Next protoCaeGroup

                End If

                If DEBUG Then theLW.WriteLine(theLWPrefix + "Owning Comp Prototype = " + objType + " tag = " + objTag.ToString + " part = " + inxobj.OwningPart.FullPath)
            Else

                If DEBUG Then theLW.WriteLine(theLWPrefix + "Owning Comp Prototype = NONE ")
            End If

            If DEBUG Then theLW.WriteLine(theLWPrefix + "Owning Comp = " + feModelOcc.GetComponent.Name)

        Else
            If DEBUG Then theLW.WriteLine(theLWPrefix + "Owning Comp = NONE ")
        End If

        If (feModelOcc.Prototype IsNot Nothing) Then
            Dim objType As String = "No Type "
            Dim objTag As Tag = Tag.Null
            If TypeOf feModelOcc.Prototype Is TaggedObject Then
                objTag = (CType(feModelOcc.Prototype, TaggedObject)).Tag
                objType = (CType(feModelOcc.Prototype, TaggedObject)).GetType.ToString
            End If

            If DEBUG Then
                theLW.WriteLine(theLWPrefix + "Prototype tag = [" + objTag.ToString + "] type =" + objType)
                theLW.WriteLine(theLWPrefix + vbTab + "Owning Part = " + feModelOcc.Prototype.OwningPart.FullPath)
            End If

        Else
            If DEBUG Then theLW.WriteLine(theLWPrefix + "No Prototype exists ")

        End If

        Dim childFeModelOccs As CAE.FEModelOccurrence() = feModelOcc.GetChildren()
        If DEBUG Then
            If childFeModelOccs Is Nothing Then
                theLW.WriteLine(theLWPrefix + vbTab + "FEModelOccurrence has no children ")
            Else
                theLW.WriteLine(theLWPrefix + vbTab + "Input FEModelOccurrence has [" + childFeModelOccs.Length.ToString() + "] children ")
            End If
        End If

        For Each childFeModelOcc As CAE.FEModelOccurrence In childFeModelOccs
            caeGroups.AddRange(processFEModelOccurrence(childFeModelOcc))
        Next

        If DEBUG Then theLW.WriteLine(vbCrLf + theLWPrefix + "processFEModelOccurrence End" + vbCrLf)

        theLWPrefix = prevPrefix

        Return caeGroups

    End Function

    Public Function GetUnloadOption(ByVal dummy As String) As Integer

        'Unloads the image when the NX session terminates
        'GetUnloadOption = NXOpen.Session.LibraryUnloadOption.AtTermination

        '----Other unload options-------
        'Unloads the image immediately after execution within NX
        GetUnloadOption = NXOpen.Session.LibraryUnloadOption.Immediately

        'Unloads the image explicitly, via an unload dialog
        'GetUnloadOption = NXOpen.Session.LibraryUnloadOption.Explicitly
        '-------------------------------

    End Function

End Module


